@@ -13,7 +13,7 @@
 ##############################################################################
 """Widget Framework Implementation
 
-$Id: widget.py 100919 2009-06-13 15:50:48Z adamg $
+$Id: widget.py 123178 2011-10-29 20:58:58Z rogerineichen $
 """
 __docformat__ = "reStructuredText"
 
@@ -21,12 +21,12 @@ import zope.interface
 import zope.component
 import zope.location
 import zope.schema.interfaces
-from zope.pagetemplate.interfaces import IPageTemplate
+from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile
 from zope.i18n import translate
+from zope.pagetemplate.interfaces import IPageTemplate
 from zope.schema.fieldproperty import FieldProperty
 
 from z3c.form import interfaces, util, value
-from z3c.form import ptcompat as viewpagetemplatefile
 
 PLACEHOLDER = object()
 
@@ -248,6 +248,7 @@ class MultiWidget(Widget):
 
     widgets = None
     _value = None
+    _widgets_updated = False
 
     _mode = FieldProperty(interfaces.IWidget['mode'])
 
@@ -269,7 +270,7 @@ class MultiWidget(Widget):
 
     @apply
     def mode():
-        """This invokes updateWidgets on any value change e.g. update/extract."""
+        """This sets the subwidgets modes."""
         def get(self):
             return self._mode
         def set(self, mode):
@@ -344,6 +345,11 @@ class MultiWidget(Widget):
         """Setup internal widgets based on the value_type for each value item.
         """
         oldLen = len(self.widgets)
+        # Ensure at least min_length widgets are shown
+        if (zope.schema.interfaces.IMinMaxLen.providedBy(self.field) and
+            self.mode == interfaces.INPUT_MODE and self.allowAdding and
+            oldLen < self.field.min_length):
+            oldLen = self.field.min_length
         self.widgets = []
         idx = 0
         if self.value:
@@ -359,6 +365,7 @@ class MultiWidget(Widget):
                 widget = self.getWidget(idx)
                 self.widgets.append(widget)
                 idx += 1
+        self._widgets_updated = True
 
     def updateAllowAddRemove(self):
         """Update the allowAdding/allowRemoving attributes
@@ -383,6 +390,13 @@ class MultiWidget(Widget):
             self.updateWidgets()
         return property(get, set)
 
+    def update(self):
+        """See z3c.form.interfaces.IWidget."""
+        # Ensure that updateWidgets is called.
+        super(MultiWidget, self).update()
+        if not self._widgets_updated:
+            self.updateWidgets()
+
     def extract(self, default=interfaces.NO_VALUE):
         # This method is responsible to get the widgets value based on the
         # request and nothing else.
@@ -401,9 +415,6 @@ class MultiWidget(Widget):
         for idx in range(counter):
             widget = self.getWidget(idx)
             append(widget.value)
-        if len(values) == 0:
-            # no multi value found
-            return interfaces.NO_VALUE
         return values
 
 
@@ -427,8 +438,7 @@ class WidgetTemplateFactory(object):
     def __init__(self, filename, contentType='text/html',
                  context=None, request=None, view=None,
                  field=None, widget=None):
-        self.template = viewpagetemplatefile.ViewPageTemplateFile(
-            filename, content_type=contentType)
+        self.template = ViewPageTemplateFile(filename, content_type=contentType)
         zope.component.adapter(
             util.getSpecification(context),
             util.getSpecification(request),
